//
// Created by trotfunky on 30/09/2019.
//

#ifndef TESTS_OPENGL_TYPES_H
#define TESTS_OPENGL_TYPES_H

#include <fstream>

/// Group of coordinates with the input operator overloaded.
/// \tparam T Content of the vector
/// \tparam N Number of coordinates
template <typename T, unsigned int N>
struct CoordinatesVector
{
    T coordinates[N];

    // Use SFINAE to declare the function if and only if the type is arithmetic
    template <typename Scalar>
    typename std::enable_if<std::is_arithmetic<Scalar>::value,CoordinatesVector<T,N>>::type
    operator*(const Scalar& scalar)
    {
        CoordinatesVector result;
        for (unsigned int i = 0;i<N;i++)
        {
            result.coordinates[i] = coordinates[i] * scalar;
        }
    }

    CoordinatesVector operator+(const CoordinatesVector<T,N>& op)
    {
        CoordinatesVector result;
        for (unsigned int i = 0;i<N;i++)
        {
            result.coordinates[i] = coordinates[i] + op.coordinates[i];
        }
        return result;
    }

    CoordinatesVector operator-(const CoordinatesVector<T,N>& op)
    {
        CoordinatesVector result;
        for (unsigned int i = 0;i<N;i++)
        {
            result.coordinates[i] = coordinates[i] - op.coordinates[i];
        }

        return result;
    }

    CoordinatesVector& operator=(const CoordinatesVector<T,N>& original)
    {
        if (this != &original)
        {
            std::copy(std::begin(original.coordinates), std::end(original.coordinates), std::begin(coordinates));
        }
        return *this;
    }

    friend std::fstream& operator>>(std::fstream& stream, CoordinatesVector<T,N>& vector)
    {
        for (unsigned int i = 0;i<N;i++)
        {
            stream >> vector.coordinates[i];
        }
        return stream;
    }
};


template <typename T>
struct CoordinatesVector<T,2>
{
    T coordinates[2];
    T& x = coordinates[0];
    T& y = coordinates[1];

    // Use SFINAE to declare the function if and only if the type is a scalar
    template <typename Scalar>
    typename std::enable_if<std::is_arithmetic<Scalar>::value,CoordinatesVector<T,2>>::type
    operator*(const Scalar& scalar)
    {
        CoordinatesVector result;
        for (unsigned int i = 0;i<2;i++)
        {
            result.coordinates[i] = coordinates[i] * scalar;
        }
    }

    CoordinatesVector operator+(const CoordinatesVector<T,2>& op)
    {
        CoordinatesVector result;
        result.x = x + op.x;
        result.y = y + op.y;

        return result;
    }

    CoordinatesVector operator-(const CoordinatesVector<T,2>& op)
    {
        CoordinatesVector<T,2> result;
        result.x = x - op.x;
        result.y = y - op.y;

        return result;
    }

    CoordinatesVector& operator=(const CoordinatesVector<T,2>& original)
    {
        if (this != &original)
        {
            std::copy(std::begin(original.coordinates), std::end(original.coordinates), std::begin(coordinates));
        }
        return *this;
    }

    friend std::fstream& operator>>(std::fstream& stream, CoordinatesVector<T,2>& vec2)
    {
        stream >> vec2.x >> vec2.y;
        return stream;
    }
};

template <typename T>
struct CoordinatesVector<T,3>
{
    T coordinates[3];

    T& x = coordinates[0];
    T& y = coordinates[1];
    T& z = coordinates[2];

    // Use SFINAE to declare the function if and only if the type is a scalar
    template <typename Scalar>
    typename std::enable_if<std::is_arithmetic<Scalar>::value,CoordinatesVector<T,3>>::type
    operator*(const Scalar& scalar)
    {
        CoordinatesVector result;
        for (unsigned int i = 0;i<3;i++)
        {
            result.coordinates[i] = coordinates[i] * scalar;
        }
    }

    CoordinatesVector operator+(const CoordinatesVector<T,3>& op)
    {
        CoordinatesVector result;
        result.x = x + op.x;
        result.y = y + op.y;
        result.z = z + op.z;

        return result;
    }

    CoordinatesVector operator-(const CoordinatesVector<T,3>& op)
    {
        CoordinatesVector<T,3> result;
        result.x = x - op.x;
        result.y = y - op.y;
        result.z = z - op.z;

        return result;
    }

    CoordinatesVector& operator=(const CoordinatesVector<T,3>& original)
    {
        if (this != &original)
        {
            std::copy(std::begin(original.coordinates), std::end(original.coordinates), std::begin(coordinates));
        }
        return *this;
    }

    friend std::fstream& operator>>(std::fstream& stream, CoordinatesVector<T,3>& vec3)
    {
        stream >> vec3.x >> vec3.y >> vec3.z;
        return stream;
    }
};


// Define aliases for common types
typedef CoordinatesVector<int,2> Vec2i;
typedef CoordinatesVector<float,2> Vec2f;

typedef CoordinatesVector<int,3> Vec3i;
typedef CoordinatesVector<float,3> Vec3f;


#endif //TESTS_OPENGL_TYPES_H
